def get_chunks(s, chunk_size, text_id):
    """
    Function to split a text, inspired by\
        https://stackoverflow.com/questions/57023348/python-splitting-a-long-text-into-chunks-of-strings-given-character-limit/57023373

    :param s: The text to chunk
    :type s: str

    :param chunk_size: Chunk size (in characters
    :type chunk_size: int

    :param text_id: Some identifier of the text, e.g the text's file name
    :type text_id: str

    :return: yields dicts like\
        `{"id": "example_text.txt", "chunk_nr": "001", "text": "text of chunk", "char_count": 13}`
    :rtype: dict
    """
    start = 0
    end = 0
    counter = 1
    while start + chunk_size < len(s) and end != -1:
        end = s.rfind(" ", start, start + chunk_size + 1)
        text = s[start:end].replace('\n', '')
        yield {
            "id": f"{counter:03}___{text_id}",
            "chunk_nr": f"{counter:03}",
            "text": text,
            "char_count": len(text)
        }
        counter += 1
        start = end + 1
    yield {
        "id": f"{counter:03}___{text_id}",
        "chunk_nr": f"{counter:03}",
        "text": s[start:].replace('\n', ''),
        "char_count": len(s[start:])
    }



#!/usr/bin/env python

"""Tests for `acdh_collatex_utils` package."""

import glob
import os
import shutil
import unittest
import lxml.etree as ET

from acdh_collatex_utils.acdh_collatex_utils import (
    chunks_to_df,
    CxReader,
    CxCollate
)

from acdh_collatex_utils.post_process import (
    make_full_tei_doc,
    merge_tei_fragments,
)


FILES = glob.glob(
    "./fixtures/*.xml",
    recursive=False
)
INPUT_FILE = "./fixtures/tmp/tmp.xml"
FINAL = "./fixtures/tmp/tmp_final.xml"
TMP_DIR = './fixtures/tmp'


class TestAcdh_collatex_utils(unittest.TestCase):
    """Tests for `acdh_collatex_utils` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_001_char_limit(self):
        """Test char_limit."""
        for x in FILES:
            doc = CxReader(xml=x, char_limit=True)
            doc_no_limit = CxReader(xml=x)
            self.assertTrue(doc.plaint_text_len <= 5000)
            self.assertTrue(doc_no_limit.plaint_text_len >= 5000)

    def test_002_clean_string(self):
        """Check if all tei:hi elments are properly removed"""
        for x in FILES:
            doc = CxReader(xml=x)
            CxReader(xml=x)
            self.assertFalse('<lb break' in f"{doc.preprocess()}")
            self.assertTrue('<lb' in f"{doc.preprocess()}")

    def test_003_chunks_to_df(self):
        df = chunks_to_df(FILES)
        self.assertTrue('id' in df.keys())

    def test_004_collate_chunks(self):
        CxCollate(output_dir='./fixtures').collate()
        new_htmls = glob.glob(
            "./fixtures/*.html",
            recursive=False
        )
        self.assertTrue(len(new_htmls) == 3)

    def test_004_merge_tei_fragments(self):
        COL_FILES = glob.glob(
            "./fixtures/*.tei",
            recursive=False
        )
        os.makedirs(TMP_DIR, exist_ok=True)
        full_doc = merge_tei_fragments(COL_FILES)
        self.assertEqual(
            "{http://www.tei-c.org/ns/1.0}ab", f"{full_doc.tag}"
        )
        with open(INPUT_FILE, 'w') as f:
            f.write(ET.tostring(full_doc, encoding='UTF-8').decode('utf-8'))
        OUT_FILES = glob.glob(
            "./fixtures/out__*.*",
            recursive=False
        )
        for x in OUT_FILES:
            os.remove(x)

    def test_005_make_full_tei_doc(self):
        full_tei = make_full_tei_doc(INPUT_FILE)
        root = full_tei.tree
        self.assertEqual(
            "{http://www.tei-c.org/ns/1.0}TEI", f"{root.tag}"
        )
        full_tei.tree_to_file(FINAL)
        shutil.rmtree(TMP_DIR)



import lxml.etree as ET
from acdh_tei_pyutils.tei import TeiReader

TEI_DUMMY_STRING = """
<?xml version="1.0" encoding="UTF-8"?>
<TEI xmlns="http://www.tei-c.org/ns/1.0">
  <teiHeader>
      <fileDesc>
         <titleStmt>
            <title>Title</title>
         </titleStmt>
         <publicationStmt>
            <p>Publication Information</p>
         </publicationStmt>
         <sourceDesc>
            <listWit/>
         </sourceDesc>
      </fileDesc>
  </teiHeader>
  <text>
      <body>
         <ab/>
      </body>
  </text>
</TEI>
"""


def merge_tei_fragments(files):
    """ takes a list of files (fullpaths) and retuns a single tei:ab element.etree node"""
    full_doc = ET.Element("{http://www.tei-c.org/ns/1.0}ab", nsmap={None: "http://www.tei-c.org/ns/1.0"})
    for x in sorted(files):
        doc = TeiReader(x)
        for rdg in doc.any_xpath('.//tei:rdg'):
            old_ids = rdg.attrib['wit'].split()
            new_ids = " ".join([f"#{x[7:]}" for x in old_ids])
            rdg.attrib['wit'] = new_ids
        for node in doc.any_xpath('./*'):
            full_doc.append(node)
    return full_doc


def make_full_tei_doc(input_file):
    """ takes the rusult of merged collated tei fragments\
        and returns a valid TEI document as TeiReader object"""
    tei_dummy = TeiReader(TEI_DUMMY_STRING)
    crit_app = TeiReader(input_file)
    body = tei_dummy.any_xpath('.//tei:ab')[0]
    list_wit_node = tei_dummy.any_xpath('.//tei:listWit')[0]
    wit_set = set()

    for rdg in crit_app.any_xpath('.//tei:rdg/@wit'):
        for w in rdg.split():
            wit_set.add(w[1:])

    for x in list(sorted(wit_set)):
        w_node = ET.Element("{http://www.tei-c.org/ns/1.0}witness", nsmap={None: "http://www.tei-c.org/ns/1.0"})
        w_node.attrib['{http://www.w3.org/XML/1998/namespace}id'] = x
        w_node.text = x
        list_wit_node.append(w_node)
    for x in crit_app.any_xpath('./*'):
        body.append(x)
    return tei_dummy
